@import "admonitions"
@import "api"
@import "blocks"
@import "captions"
@import "code"
@import "footnotes"
@import "images"
@import "indexes"
@import "lists"
@import "math"
@import "misc"
@import "rubrics"
@import "sidebar"
@import "tables"
@import "target"
@import "gui-labels"
