// This file contains styles to tweak sphinx-inline-tabs to work well with Furo.

body
  --tabs--label-text: var(--color-foreground-muted)
  --tabs--label-text--hover: var(--color-foreground-muted)
  --tabs--label-text--active: var(--color-brand-content)
  --tabs--label-text--active--hover: var(--color-brand-content)
  --tabs--label-background: transparent
  --tabs--label-background--hover: transparent
  --tabs--label-background--active: transparent
  --tabs--label-background--active--hover: transparent
  --tabs--padding-x: 0.25em
  --tabs--margin-x: 1em
  --tabs--border: var(--color-background-border)
  --tabs--label-border: transparent
  --tabs--label-border--hover: var(--color-foreground-muted)
  --tabs--label-border--active: var(--color-brand-content)
  --tabs--label-border--active--hover: var(--color-brand-content)
